@@ -27,15 +27,12 @@
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
+import asyncio
 import difflib
-import tempfile
-import subprocess
 import re
-import shlex
 import email.utils
 import itertools
 import logging
-import os
 import sys
 import time
 import html
@@ -46,7 +43,8 @@ import requests
 import urlwatch
 from .mailer import SMTPMailer
 from .mailer import SendmailMailer
-from .util import TrackSubClasses
+from .util import TrackSubClasses, chunkstring
+from .xmpp import XMPP
 
 try:
     import chump
@@ -69,6 +67,11 @@ try:
 except ImportError:
     Markdown = None
 
+try:
+    from colorama import AnsiToWin32
+except ImportError:
+    AnsiToWin32 = None
+
 logger = logging.getLogger(__name__)
 
 # Regular expressions that match the added/removed markers of GNU wdiff output
@@ -102,6 +105,15 @@ class ReporterBase(object, metaclass=TrackSubClasses):
             ))
         return '\n'.join(result)
 
+    @classmethod
+    def submit_one(cls, name, report, job_states, duration):
+        subclass = cls.__subclasses__[name]
+        cfg = report.config['report'].get(name, {'enabled': False})
+        if cfg['enabled']:
+            subclass(report, cfg, job_states, duration).submit()
+        else:
+            raise ValueError('Reporter not enabled: {name}'.format(name=name))
+
     @classmethod
     def submit_all(cls, report, job_states, duration):
         any_enabled = False
@@ -113,34 +125,11 @@ class ReporterBase(object, metaclass=TrackSubClasses):
                 subclass(report, cfg, job_states, duration).submit()
 
         if not any_enabled:
-            logger.warn('No reporters enabled.')
+            logger.warning('No reporters enabled.')
 
     def submit(self):
         raise NotImplementedError()
 
-    def unified_diff(self, job_state):
-        if job_state.job.diff_tool is not None:
-            with tempfile.TemporaryDirectory() as tmpdir:
-                old_file_path = os.path.join(tmpdir, 'old_file')
-                new_file_path = os.path.join(tmpdir, 'new_file')
-                with open(old_file_path, 'w+b') as old_file, open(new_file_path, 'w+b') as new_file:
-                    old_file.write(job_state.old_data.encode('utf-8'))
-                    new_file.write(job_state.new_data.encode('utf-8'))
-                cmdline = shlex.split(job_state.job.diff_tool) + [old_file_path, new_file_path]
-                proc = subprocess.Popen(cmdline, stdout=subprocess.PIPE)
-                stdout, _ = proc.communicate()
-                # Diff tools return 0 for "nothing changed" or 1 for "files differ", anything else is an error
-                if proc.returncode in (0, 1):
-                    return stdout.decode('utf-8')
-                else:
-                    raise subprocess.CalledProcessError(proc.returncode, cmdline)
-
-        timestamp_old = email.utils.formatdate(job_state.timestamp, localtime=1)
-        timestamp_new = email.utils.formatdate(time.time(), localtime=1)
-        return ''.join(difflib.unified_diff([l + '\n' for l in job_state.old_data.splitlines()],
-                                            [l + '\n' for l in job_state.new_data.splitlines()],
-                                            '@', '@', timestamp_old, timestamp_new))
-
 
 class SafeHtml(object):
     def __init__(self, s):
@@ -232,15 +221,16 @@ class HtmlReporter(ReporterBase):
             return SafeHtml('...')
 
         if difftype == 'table':
-            timestamp_old = email.utils.formatdate(job_state.timestamp, localtime=1)
-            timestamp_new = email.utils.formatdate(time.time(), localtime=1)
+            timestamp_old = email.utils.formatdate(job_state.timestamp, localtime=True)
+            timestamp_new = email.utils.formatdate(time.time(), localtime=True)
             html_diff = difflib.HtmlDiff()
-            return SafeHtml(html_diff.make_table(job_state.old_data.splitlines(1), job_state.new_data.splitlines(1),
+            return SafeHtml(html_diff.make_table(job_state.old_data.splitlines(keepends=True),
+                                                 job_state.new_data.splitlines(keepends=True),
                                                  timestamp_old, timestamp_new, True, 3))
         elif difftype == 'unified':
             return ''.join((
                 '<pre>',
-                '\n'.join(self._diff_to_html(self.unified_diff(job_state))),
+                '\n'.join(self._diff_to_html(job_state.get_diff())),
                 '</pre>',
             ))
         else:
@@ -297,7 +287,7 @@ class TextReporter(ReporterBase):
         if job_state.old_data in (None, job_state.new_data):
             return None
 
-        return self.unified_diff(job_state)
+        return job_state.get_diff()
 
     def _format_output(self, job_state, line_length):
         summary_part = []
@@ -351,8 +341,7 @@ class StdoutReporter(TextReporter):
         return self._incolor(4, s)
 
     def _get_print(self):
-        if sys.platform == 'win32' and self._has_color:
-            from colorama import AnsiToWin32
+        if sys.platform == 'win32' and self._has_color and AnsiToWin32 is not None:
             return functools.partial(print, file=AnsiToWin32(sys.stdout).stream)
         return print
 
@@ -410,7 +399,7 @@ class EMailReporter(TextReporter):
             smtp_user = self.config['smtp'].get('user', None) or self.config['from']
             # Legacy support: The current smtp "auth" setting was previously called "keyring"
             if 'keyring' in self.config['smtp']:
-                logger.info('The SMTP config key "keyring" is now called "auth".')
+                logger.info('The SMTP config key "keyring" is now called "auth". See https://urlwatch.readthedocs.io/en/latest/deprecated.html')
             use_auth = self.config['smtp'].get('auth', self.config['smtp'].get('keyring', False))
             mailer = SMTPMailer(smtp_user, self.config['smtp']['host'], self.config['smtp']['port'],
                                 self.config['smtp']['starttls'], use_auth,
@@ -430,6 +419,23 @@ class EMailReporter(TextReporter):
         mailer.send(msg)
 
 
+class IFTTTReport(TextReporter):
+    """Send summary via IFTTT"""
+
+    __kind__ = 'ifttt'
+
+    def submit(self):
+        webhook_url = 'https://maker.ifttt.com/trigger/{event}/with/key/{key}'.format(**self.config)
+        for job_state in self.report.get_filtered_job_states(self.job_states):
+            pretty_name = job_state.job.pretty_name()
+            location = job_state.job.get_location()
+            result = requests.post(webhook_url, json={
+                'value1': job_state.verb,
+                'value2': pretty_name,
+                'value3': location,
+            })
+
+
 class WebServiceReporter(TextReporter):
     MAX_LENGTH = 1024
 
@@ -451,7 +457,7 @@ class WebServiceReporter(TextReporter):
 
         try:
             service = self.web_service_get()
-        except Exception as e:
+        except Exception:
             logger.error('Failed to load or connect to %s - are the dependencies installed and configured?',
                          self.__kind__, exc_info=True)
             return
@@ -465,6 +471,9 @@ class PushoverReport(WebServiceReporter):
     __kind__ = 'pushover'
 
     def web_service_get(self):
+        if chump is None:
+            raise ImportError('Python module "chump" not installed')
+
         app = chump.Application(self.config['app'])
         return app.get_user(self.config['user'])
 
@@ -473,7 +482,14 @@ class PushoverReport(WebServiceReporter):
         # If device is the empty string or not specified at all, use None to send to all devices
         # (see https://github.com/thp/urlwatch/issues/372)
         device = self.config.get('device', None) or None
-        msg = service.create_message(title=title, message=body, html=True, sound=sound, device=device)
+        priority = {
+            'lowest': chump.LOWEST,
+            'low': chump.LOW,
+            'normal': chump.NORMAL,
+            'high': chump.HIGH,
+            'emergency': chump.EMERGENCY,
+        }.get(self.config.get('priority', None), chump.NORMAL)
+        msg = service.create_message(title=title, message=body, html=True, sound=sound, device=device, priority=priority)
         msg.send()
 
 
@@ -483,6 +499,9 @@ class PushbulletReport(WebServiceReporter):
     __kind__ = 'pushbullet'
 
     def web_service_get(self):
+        if Pushbullet is None:
+            raise ImportError('Python module "pushbullet" not installed')
+
         return Pushbullet(self.config['api_key'])
 
     def web_service_submit(self, service, title, body):
@@ -490,7 +509,7 @@ class PushbulletReport(WebServiceReporter):
 
 
 class MailGunReporter(TextReporter):
-    """Custom email reporter that uses Mailgun"""
+    """Send e-mail via the Mailgun service"""
 
     __kind__ = 'mailgun'
 
@@ -548,7 +567,7 @@ class MailGunReporter(TextReporter):
 
 
 class TelegramReporter(TextReporter):
-    """Custom Telegram reporter"""
+    """Send a message using Telegram"""
     MAX_LENGTH = 4096
 
     __kind__ = 'telegram'
@@ -566,7 +585,7 @@ class TelegramReporter(TextReporter):
             return
 
         result = None
-        for chunk in self.chunkstring(text, self.MAX_LENGTH):
+        for chunk in chunkstring(text, self.MAX_LENGTH, numbering=True):
             for chat_id in chat_ids:
                 res = self.submitToTelegram(bot_token, chat_id, chunk)
                 if res.status_code != requests.codes.ok or res is None:
@@ -592,12 +611,9 @@ class TelegramReporter(TextReporter):
                                                                                                 result.content))
         return result
 
-    def chunkstring(self, string, length):
-        return (string[0 + i:length + i] for i in range(0, len(string), length))
-
 
 class SlackReporter(TextReporter):
-    """Custom Slack reporter"""
+    """Send a message to a Slack channel"""
     MAX_LENGTH = 40000
 
     __kind__ = 'slack'
@@ -611,7 +627,7 @@ class SlackReporter(TextReporter):
             return
 
         result = None
-        for chunk in self.chunkstring(text, self.MAX_LENGTH):
+        for chunk in chunkstring(text, self.MAX_LENGTH, numbering=True):
             res = self.submit_to_slack(webhook_url, chunk)
             if res.status_code != requests.codes.ok or res is None:
                 result = res
@@ -633,9 +649,6 @@ class SlackReporter(TextReporter):
                                                                                              result.content))
         return result
 
-    def chunkstring(self, string, length):
-        return (string[0 + i:length + i] for i in range(0, len(string), length))
-
 
 class MarkdownReporter(ReporterBase):
     def submit(self):
@@ -682,7 +695,7 @@ class MarkdownReporter(ReporterBase):
         if job_state.old_data in (None, job_state.new_data):
             return None
 
-        return self.unified_diff(job_state)
+        return job_state.get_diff()
 
     def _format_output(self, job_state):
         summary_part = []
@@ -708,12 +721,15 @@ class MarkdownReporter(ReporterBase):
 
 
 class MatrixReporter(MarkdownReporter):
-    """Custom Matrix reporter"""
+    """Send a message to a room using the Matrix protocol"""
     MAX_LENGTH = 4096
 
     __kind__ = 'matrix'
 
     def submit(self):
+        if matrix_client is None:
+            raise ImportError('Python module "matrix_client" not installed')
+
         homeserver_url = self.config['homeserver']
         access_token = self.config['access_token']
         room_id = self.config['room_id']
@@ -745,3 +761,26 @@ class MatrixReporter(MarkdownReporter):
         else:
             logger.debug('Not formatting as Markdown; dependency on markdown2 not met?')
             client_api.send_message(room_id, body_markdown)
+
+
+class XMPPReporter(TextReporter):
+    """Send a message using the XMPP Protocol"""
+    MAX_LENGTH = 262144
+
+    __kind__ = 'xmpp'
+
+    def submit(self):
+
+        sender = self.config['sender']
+        recipient = self.config['recipient']
+
+        text = '\n'.join(super().submit())
+
+        if not text:
+            logger.debug('Not sending XMPP message (no changes)')
+            return
+
+        xmpp = XMPP(sender, recipient, self.config.get('insecure_password'))
+
+        for chunk in chunkstring(text, self.MAX_LENGTH, numbering=True):
+            asyncio.run(xmpp.send(chunk))
